@@ -5,6 +5,7 @@ from twisted.python import log
 from twisted.persisted import styles
 from twisted.internet import reactor, defer
 from twisted.protocols import basic
+from buildbot.process.properties import Properties
 
 import os, shutil, sys, re, urllib, itertools
 from cPickle import load, dump
@@ -448,7 +449,6 @@ class LogFile:
             self.finish() # releases self.openfile, which will be closed
         del self.entries
 
-
 class HTMLLogFile:
     implements(interfaces.IStatusLog)
 
@@ -641,6 +641,11 @@ class BuildStepStatus(styles.Versioned):
     I represent a collection of output status for a
     L{buildbot.process.step.BuildStep}.
 
+    Statistics contain any information gleaned from a step that is
+    not in the form of a logfile.  As an example, steps that run
+    tests might gather statistics about the number of passed, failed,
+    or skipped tests.
+
     @type color: string
     @cvar color: color that this step feels best represents its
                  current mood. yellow,green,red,orange are the
@@ -654,11 +659,13 @@ class BuildStepStatus(styles.Versioned):
     @cvar text2: list of short texts added to the overall build description
     @type logs: dict of string -> L{buildbot.status.builder.LogFile}
     @ivar logs: logs of steps
+    @type statistics: dict
+    @ivar statistics: results from running this step
     """
     # note that these are created when the Build is set up, before each
     # corresponding BuildStep has started.
     implements(interfaces.IBuildStepStatus, interfaces.IStatusEvent)
-    persistenceVersion = 1
+    persistenceVersion = 2
 
     started = None
     finished = None
@@ -670,6 +677,7 @@ class BuildStepStatus(styles.Versioned):
     watchers = []
     updates = {}
     finishedWatchers = []
+    statistics = {}
 
     def __init__(self, parent):
         assert interfaces.IBuildStatus(parent)
@@ -679,6 +687,7 @@ class BuildStepStatus(styles.Versioned):
         self.watchers = []
         self.updates = {}
         self.finishedWatchers = []
+        self.statistics = {}
 
     def getName(self):
         """Returns a short string with the name of this step. This string
@@ -763,6 +772,16 @@ class BuildStepStatus(styles.Versioned):
         """
         return (self.results, self.text2)
 
+    def hasStatistic(self, name):
+        """Return true if this step has a value for the given statistic.
+        """
+        return self.statistics.has_key(name)
+
+    def getStatistic(self, name, default=None):
+        """Return the given statistic, if present
+        """
+        return self.statistics.get(name, default)
+
     # subscription interface
 
     def subscribe(self, receiver, updateInterval=10):
@@ -846,6 +865,11 @@ class BuildStepStatus(styles.Versioned):
     def setText2(self, text):
         self.text2 = text
 
+    def setStatistic(self, name, value):
+        """Set the given statistic.  Usually called by subclasses.
+        """
+        self.statistics[name] = value
+
     def stepFinished(self, results):
         self.finished = util.now()
         self.results = results
@@ -885,10 +909,14 @@ class BuildStepStatus(styles.Versioned):
         if not hasattr(self, "urls"):
             self.urls = {}
 
+    def upgradeToVersion2(self):
+        if not hasattr(self, "statistics"):
+            self.statistics = {}
+
 
 class BuildStatus(styles.Versioned):
     implements(interfaces.IBuildStatus, interfaces.IStatusEvent)
-    persistenceVersion = 2
+    persistenceVersion = 3
 
     source = None
     reason = None
@@ -924,7 +952,7 @@ class BuildStatus(styles.Versioned):
         self.finishedWatchers = []
         self.steps = []
         self.testResults = {}
-        self.properties = {}
+        self.properties = Properties()
 
     # IBuildStatus
 
@@ -937,6 +965,9 @@ class BuildStatus(styles.Versioned):
     def getProperty(self, propname):
         return self.properties[propname]
 
+    def getProperties(self):
+        return self.properties
+
     def getNumber(self):
         return self.number
 
@@ -974,6 +1005,23 @@ class BuildStatus(styles.Versioned):
     def getTimes(self):
         return (self.started, self.finished)
 
+    _sentinel = [] # used as a sentinel to indicate unspecified initial_value
+    def getSummaryStatistic(self, name, summary_fn, initial_value=_sentinel):
+        """Summarize the named statistic over all steps in which it
+        exists, using combination_fn and initial_value to combine multiple
+        results into a single result.  This translates to a call to Python's
+        X{reduce}::
+            return reduce(summary_fn, step_stats_list, initial_value)
+        """
+        step_stats_list = [
+                st.getStatistic(name)
+                for st in self.steps
+                if st.hasStatistic(name) ]
+        if initial_value is self._sentinel:
+            return reduce(summary_fn, step_stats_list)
+        else:
+            return reduce(summary_fn, step_stats_list, initial_value)
+
     def isFinished(self):
         return (self.finished is not None)
 
@@ -1074,8 +1122,8 @@ class BuildStatus(styles.Versioned):
         self.steps.append(s)
         return s
 
-    def setProperty(self, propname, value):
-        self.properties[propname] = value
+    def setProperty(self, propname, value, source):
+        self.properties.setProperty(propname, value, source)
 
     def addTestResult(self, result):
         self.testResults[result.getName()] = result
@@ -1229,6 +1277,12 @@ class BuildStatus(styles.Versioned):
     def upgradeToVersion2(self):
         self.properties = {}
 
+    def upgradeToVersion3(self):
+        # in version 3, self.properties became a Properties object
+        propdict = self.properties
+        self.properties = Properties()
+        self.properties.update(propdict, "Upgrade from previous version")
+
     def upgradeLogfiles(self):
         # upgrade any LogFiles that need it. This must occur after we've been
         # attached to our Builder, and after we know about all LogFiles of
@@ -1733,6 +1787,7 @@ class SlaveStatus:
     def __init__(self, name):
         self.name = name
         self._lastMessageReceived = 0
+        self.runningBuilds = []
 
     def getName(self):
         return self.name
@@ -1744,6 +1799,8 @@ class SlaveStatus:
         return self.connected
     def lastMessageReceived(self):
         return self._lastMessageReceived
+    def getRunningBuilds(self):
+        return self.runningBuilds
 
     def setAdmin(self, admin):
         self.admin = admin
@@ -1754,6 +1811,11 @@ class SlaveStatus:
     def setLastMessageReceived(self, when):
         self._lastMessageReceived = when
 
+    def buildStarted(self, build):
+        self.runningBuilds.append(build)
+    def buildFinished(self, build):
+        self.runningBuilds.remove(build)
+
 class Status:
     """
     I represent the status of the buildmaster.
